@@ -51,6 +51,8 @@
  */
 
 
+#include "config.h"
+
 #include "afflib.h"			
 #include "afflib_i.h"			// grab the internal versions
 #include "aimage.h"
@@ -101,7 +103,7 @@ char *command_line = 0;			// what is typed
 int  opt_quiet = 0;
 int  opt_silent = 0;
 int  opt_beeps  = 1;			// beep when done
-int  opt_error_count = 0;
+int  opt_error_mode = 0;
 int  opt_retry_count = 5;
 int  opt_reverse = 0;
 int  opt_fast_quit = 0;
@@ -219,7 +221,7 @@ void usage()
 
 
     bold("\nError Recovery Options:\n");
-    printf("  --error=0, -e0  -- Standard error recovery:\n");
+    printf("  --error_mode=0, -e0  -- Standard error recovery:\n");
     printf("                     Read disk 256K at a time until there are 5 errors in a row.\n");
     printf("                     Then go to the end of the disk and read backwards\n");
     printf("                     until there are 5 erros in a row. Then stop.\n");
@@ -242,6 +244,12 @@ void usage()
     printf("\n");
     printf("Create image0.aff from /dev/sd0 and image1 from /dev/sd1:\n");
     printf("     aimage /dev/sd0 image0.aff /dev/sd1 /image1.aff\n");
+    printf("\n");
+    printf("Default values:\n");
+    printf("   config file: %s (need not be present)\n",AIMAGE_CONFIG_FILENAME);
+    printf("   page size: %d\n",default_pagesize);
+    printf("   default compression: %d\n",AF_COMPRESSION_DEFAULT);
+    printf("   dfault compression algorithm: ZLIB\n");
     exit(0);
 }
 
@@ -280,6 +288,7 @@ static struct option longopts[] = {
     { "no_preview",    no_argument,        NULL, 'P'},
     { "maxsize",       required_argument,  NULL, 'M'},
     { "debug",         required_argument,  NULL, 'd'},
+    { "error_mode",    required_argument,  NULL, 'e'},
     { "setseg",        required_argument,  NULL, 's'},
     { "no_hash",       no_argument,        NULL, 'H'},
     { "use_timers",    no_argument,        NULL, 'y'},
@@ -287,6 +296,7 @@ static struct option longopts[] = {
     { "no_dmesg",      no_argument,        NULL, 'D'},
     { "no_ifconfig",   no_argument,        NULL, 'I'},
     { "batch",         no_argument,        NULL, 'Y'},
+    { "skip",          required_argument,  NULL, 'k'},
     {0,0,0,0}
 };
 
@@ -416,7 +426,7 @@ void sig_intr(int arg)
 	af_set_callback(af,0);
 	af_enable_compression(af, 0, 0);
 	if(af_close(af)){
-	    warn("Can't close file '%s'\n",af_filename(af));
+	    warnx("Can't close file '%s'\n",af_filename(af));
 	}
     }
     fflush(stdout);
@@ -642,7 +652,7 @@ void process_option(class imager *im,char ch,char *optarg)
     case 'B': opt_beeps = 0;		break;
     case 'd': opt_debug = atoi(optarg); if(opt_debug==0) debug_list(); break;
     case 'D': opt_no_dmesg=1;break;
-    case 'e': opt_error_count = atoi(optarg); break;
+    case 'e': opt_error_mode = atoi(optarg); break;
     case 'H': im->hash_invalid = 1;    break; // don't calculate the hash
     case 'I': opt_no_ifconfig=1;break;
     case 'r': strcpy(im->fname_raw,optarg);	break;
@@ -674,7 +684,7 @@ void process_option(class imager *im,char ch,char *optarg)
 	break;
 
     case 'v':
-	printf("aimage %s\n\n",xstr(PACKAGE_VERSION));
+	printf("aimage %s\n\n",PACKAGE_VERSION);
 	exit(0);
     case 'X': opt_compression_level = atoi(optarg); break;
     case 't': opt_retry_count = atoi(optarg); break;
@@ -933,7 +943,7 @@ int main(int argc,char **argv)
 	/* Raw Cleanup */
 	if(im->fout){
 	    if(fclose(im->fout)){
-		warn("fclose(%s)",im->fname_raw);
+		warnx("fclose(%s)",im->fname_raw);
 		fprintf(stderr,"This shouldn't happen. '%s' may be corrupt.\n", im->fname_raw);
 	    }
 	    im->fout = 0;
@@ -943,7 +953,7 @@ int main(int argc,char **argv)
 	if(im->af){
 	    if(opt_debug==2) fprintf(stderr,"af->bytes_memcpy=%qd\n",im->af->bytes_memcpy);
 	    if(af_close(im->af)){
-		warn("af_close failed. This shouldn't happen.\n");
+		warnx("af_close failed. This shouldn't happen.\n");
 		fprintf(stderr,"Run 'ainfo -v %s' to see if %s is corrupt.\n",
 			im->fname_aff,im->fname_aff);
 	    }
