@@ -1,3 +1,4 @@
+#include "config.h"
 #include "afflib.h"
 #include "afflib_i.h"
 #include "imager.h"
@@ -15,10 +16,19 @@
 #include <fcntl.h>
 #include <string.h>
 #include <signal.h>
-#include <term.h>
 #include <assert.h>
 #include <errno.h>
 
+#ifdef HAVE_TERM_H
+#include <term.h>
+#endif
+
+#ifdef HAVE_NCURSES_TERM_H
+#include <ncurses/term.h>
+#endif
+
+
+
 /*
  * imager.cpp:
  * The C++ imaging library.
@@ -28,7 +38,6 @@ imager::imager()
 {
     allow_regular = false;
     total_segments_written = 0;
-    total_sectors_written = 0;
     total_sectors_read=0;
     total_bytes_read = 0;
     total_bytes_written = 0;
@@ -177,7 +186,6 @@ void imager::write_data(unsigned char *buf,uint64 offset,int len)
 	    exit(1);
 	}
     }
-    total_sectors_written += len/sector_size;
     total_bytes_written   += len;
 }
 
@@ -267,7 +275,7 @@ int  imager::open_dev(const char *friendly_name)
 			/* The device was successfully opened. */
 			return fd;		// got it!
 		    }
-		    perror(infile);		// give a warning
+		    perror(infile);	       
 		}
 	    }
 	    printf("Detaching device and trying again...\n");
@@ -323,6 +331,7 @@ void imager::image_loop(uint64 low_water_mark, // sector # to start
     uint64 data_offset = 0;		// offset into output file
     bool valid_reverse_data = false;		// did we ever get valid data in the reverse direction?
     bool last_read_short = false;
+    int reminder = 0;
 
     if(!buf) err(1,"malloc");
 
@@ -368,7 +377,9 @@ void imager::image_loop(uint64 low_water_mark, // sector # to start
 	last_sectors_read = sectors_to_read;
 	last_direction = direction;
 
-	data_offset = sector_size * snum; // where we want to start reading
+	if (high_water_mark != 0) // otherwise just sum
+		data_offset = sector_size * snum; // where we want to start reading
+
 
 	if(data_offset != in_pos){	// eliminate unnecessary seeks
 	    lseek(in,data_offset,SEEK_SET); // make sure we are at the right place; (ignore error)
@@ -439,7 +450,8 @@ void imager::image_loop(uint64 low_water_mark, // sector # to start
 	    total_bytes_read      += bytes_read; 
 	    write_data(buf,data_offset,bytes_read);
 	    data_offset += bytes_read;	// move along
-	    low_water_mark += bytes_read/sector_size;
+	    low_water_mark += (bytes_read + reminder)/sector_size;
+	    reminder = (bytes_read + reminder)%sector_size;
 	    last_read_short = true;
 	    continue;
 	}
@@ -645,7 +657,7 @@ void imager::start_imaging2()
     hash_setup();		// get ready...
     image_loop(opt_skip,
 	       total_sectors,starting_direction,
-	       opt_readsectors,opt_error_count); // start the process
+	       opt_readsectors,opt_error_mode); // start the process
     signal(SIGINT,0);		// unset the handler
 
 
@@ -776,7 +788,7 @@ int imager::socket_listen(int port)
     int sock = socket(AF_INET,SOCK_STREAM,IPPROTO_IP);    /* Open a listening socket ... */
     memset(&local,0,sizeof(local));
     memset(&remote,0,sizeof(remote));
-#ifndef linux				// have I mentioned how much I hate linux?
+#ifdef HAVE_SOCKADDR_SIN_LEN
     local.sin_len = sizeof(sockaddr_in);
 #endif
     local.sin_family = AF_INET;
@@ -827,13 +839,10 @@ void imager::final_report()
     
     char buf[64];
 
-    printf("  Bytes read: %s\n",
-	   af_commas(buf,total_bytes_read));
+    printf("  Bytes read: %s\n", af_commas(buf,total_bytes_read));
 
-    printf("  Bytes written: %s\n",
-	   af_commas(buf,total_bytes_written));
+    printf("  Bytes written: %s\n", af_commas(buf,callback_bytes_written));
 
-    
     char print_buf[256];
     printf("\n");
     if(hash_invalid==false){
