@@ -54,7 +54,7 @@ int	af_toc_build(AFFILE *af)	// build the dir if we couldn't find it
 {
     af_toc_free(af);
     af_rewind_seg(af);			// start at the beginning
-    af->toc = (af_toc_mem *)malloc(0);
+    af->toc = (af_toc_mem *)malloc(sizeof(af_toc_mem));	// was malloc(0), but that caused problems under Borland
     while(1){
 	char segname[AF_MAX_NAME_LEN];
 	size_t segname_len=sizeof(segname);
