@@ -47,10 +47,7 @@
 #include "vnode_afm.h"
 #include "vnode_aff.h"
 #include "vnode_afd.h"
-#ifdef UNIX
-#include "vnode_evf.h"
-#include "vnode_evd.h"
-#endif
+#include "vnode_ewf.h"
 
 #ifdef USE_S3
 #include "vnode_s3.h"
@@ -58,7 +55,7 @@
 
 #include <fcntl.h>
 #include <assert.h>
-#include <openssl/rand.h>
+//#include <openssl/rand.h>
 #include <errno.h>
 
 // vnode implementations.
@@ -70,9 +67,8 @@ struct af_vnode *af_vnode_array[] = {
     &vnode_afd, 
     &vnode_afm,				// must be before aff
     &vnode_aff,
-#ifdef UNIX
-    &vnode_evd,
-    &vnode_evf,
+#ifdef USE_LIBEWF
+    &vnode_ewf,				// libewf
 #endif
     &vnode_split_raw,			// must be before raw
     &vnode_raw,				// greedy; must be last
@@ -122,7 +118,12 @@ void af_initialize()
     if(val) af_cache_debug = atoi(val);
 
     val = getenv(AFFLIB_TRACE);
-    if(val) af_trace = stderr;
+    if(val){
+	af_trace = fopen(val,"wa");
+	fprintf(af_trace,"============================\n");
+	fprintf(af_trace,"AFFLIB trace started\n");
+	setvbuf(af_trace,0,_IONBF,0);
+    }
     aff_initialized = 1;
     
 }
@@ -155,27 +156,18 @@ const char *af_identify_file_name(const char *filename,int exists)
     return 0;
 }
 
-#include <stdarg.h>
-void aff_warnx(const char *fmt,...)
-{
-  va_list ap;
-  va_start(ap,fmt);
-  vfprintf(stderr,fmt,ap);
-  va_end(ap);
-}
-
 AFFILE *af_open_with(const char *filename,int flags,int mode, struct af_vnode *v)
 {
     /* Alloate the space for the AFFILE structure */
     AFFILE *af = (AFFILE *)calloc(sizeof(AFFILE),1);
     af->v	  = v;
     af->version   = 2;
-    af->openflags = flags;
+    af->openflags = flags | O_BINARY;	// make sure that we ask for binray
     af->fname	  = strdup(filename);	                // remember file name
     af->exists    = (access(filename,R_OK) == 0);	// does the file exist?
     af->openmode  = mode;
     af->image_sectorsize = 512;		// default size
-    af->error_reporter = aff_warnx;
+    af->error_reporter = warnx;
     af->badflag   = (unsigned char *)malloc(af->image_sectorsize);
 
     /* Right now just set up the cache by hand */
@@ -187,11 +179,15 @@ AFFILE *af_open_with(const char *filename,int flags,int mode, struct af_vnode *v
     /* Try opening it! */
     if((*af->v->open)(af)){		
 	/* Got an error; Free what was allocated and return */
-	free(af->fname);
+	if(af->badflag) free(af->badflag);
+	if(af->pbcache) free(af->pbcache);
+	if(af->fname) free(af->fname);
+	memset(af,0,sizeof(*af));	// clean object reuse.
 	free(af);
 	return 0;
     }
     af_read_sizes(af);			// set up the metadata
+    if(af_trace) fprintf(af_trace,"af_open_with(%s,%o,%o,%s)\n",filename,flags,mode,v->name);
     return af;
 }
 
@@ -277,6 +273,7 @@ int af_close(AFFILE *af)
     }
     if(af->fname)	free(af->fname);
     if(af->badflag)	free(af->badflag);
+    if(af->toc)         free(af->toc);
     memset(af,0,sizeof(*af));		// clean object reuse
     free(af);
     return ret;
@@ -296,7 +293,7 @@ uint64 af_seek(AFFILE *af,int64 pos,int whence)
 	new_pos += pos;
 	break;
     case SEEK_END:
-	new_pos = af_imagesize(af) - pos;
+	new_pos = af_get_imagesize(af) - pos;
 	break;
     }
     if(new_pos < 0) new_pos = 0;	   // new change
@@ -316,13 +313,13 @@ int af_eof(AFFILE *af)
 {
     af_vnode_info vni;
 
-    if(af_vstat(af,&vni)) return -1;	// that's bad
+    if(af_vstat(af,&vni)) return -1;	// this is bad; we need vstat...
     if(vni.use_eof) return vni.at_eof;	// if implementation wants to use it, use it
-    if(af->pos<0){
+    if(af->pos<0){			// pos shouldn't be <0
 	errno = EINVAL;
 	return -1;		// this is bad
     }
-    return (int64)af->pos >= af_imagesize(af);
+    return (int64)af->pos >= vni.imagesize;
 }
 
 void af_enable_writing(AFFILE *af,int flag)
@@ -376,10 +373,10 @@ int	af_identify(AFFILE *af)
     return af->v->type;
 }
 
-/* af_imagesize:
+/* af_get_imagesize:
  * Return the byte # of last mapped byte in image, or size of device;
  */
-int64       af_imagesize(AFFILE *af)
+int64       af_get_imagesize(AFFILE *af)
 {
     struct af_vnode_info vni;
     if(af_vstat(af,&vni)==0){
@@ -469,7 +466,7 @@ void af_stats(AFFILE *af,FILE *f)
     
 }
 
-#ifdef NO_ERR
+#ifndef HAVE_ERR
 #include <stdarg.h>
 void err(int eval,const char *fmt,...)
 {
@@ -480,6 +477,10 @@ void err(int eval,const char *fmt,...)
   va_end(ap);
   exit(eval);
 }
+#endif
+
+#ifndef HAVE_ERRX
+#include <stdarg.h>
 void errx(int eval,const char *fmt,...)
 {
   va_list ap;
@@ -491,6 +492,28 @@ void errx(int eval,const char *fmt,...)
 }
 #endif
 
+#ifndef HAVE_WARN
+#include <stdarg.h>
+void	warn(const char *fmt, ...)
+{
+    va_list args;
+    va_start(args,fmt);
+    vfprintf(stderr,fmt, args);
+    fprintf(stderr,": %s",strerror(errno));
+}
+#endif
+
+#ifndef HAVE_WARNX
+#include <stdarg.h>
+void warnx(const char *fmt,...)
+{
+  va_list ap;
+  va_start(ap,fmt);
+  vfprintf(stderr,fmt,ap);
+  va_end(ap);
+}
+#endif
+
 
 
 
