@@ -11,6 +11,16 @@
 
 /* Figure out what kind of OS we are running on */
 
+/* Check to make sure config.h was included */
+
+/* These are both needed; no need to bother with config.h #defines */
+#include <stdio.h>
+#include <sys/types.h>
+
+#ifdef HAVE_SYS_CDEFS_H
+#include <sys/cdefs.h>
+#endif
+
 #ifdef __FreeBSD__
 #define BSD_LIKE
 #endif
@@ -28,15 +38,15 @@
 #endif
 
 #ifdef BSD_LIKE
+#define UNIX
 typedef unsigned long long uint64;
 typedef long long int64;
 #define I64d "qd"
 #define I64u "qu"
-#include <sys/cdefs.h>
-#define UNIX
 #endif
 
 #ifdef linux
+#define UNIX
 typedef unsigned long long uint64;
 typedef long long  int64;
 #define I64d "qd"
@@ -44,17 +54,17 @@ typedef long long  int64;
 /* Horrible lossage stuff for largefile support under Linux */
 #define _LARGEFILE_SOURCE 1
 #define _FILE_OFFSET_BITS 64
-#include <sys/cdefs.h>
-#define UNIX
 #endif
 
 #ifdef sun
-#include <inttypes.h>
 typedef unsigned long long uint64;
 typedef long long  int64;
 #define I64d "qd"
 #define I64u "qu"
 #define UNIX
+#ifdef HAVE_INTTYPES_H
+#include <inttypes.h>
+#endif
 #endif 
 
 #ifdef __CYGWIN__
@@ -66,8 +76,6 @@ typedef long long  int64;
 #endif
 
 
-#include <stdio.h>
-
 
 /* WIN32 is defined by the NMAKE makefile for Visual C++ under Windows */
 #ifdef WIN32                            
@@ -93,17 +101,17 @@ typedef long long int64;
 #define UNIX
 #endif 
 
-#ifdef UNIX
 #define USE_LZMA
-#endif
 
 struct affcallback_info;
 struct aff_pagebuf {
     int64         pagenum;		// -1 means no page loaded 
     unsigned char *pagebuf;		// where the data is; size is image_pagesize
     size_t        pagebuf_bytes;        // number of bytes in the pagebuf that are valid.
-    unsigned int  dirty:1;		// is buffer dirty?
-    unsigned int  valid:1;		// is buffer dirty?
+    unsigned int  pagenum_valid:1;	// buffer contains data
+    unsigned int  pagebuf_valid:1;	// buffer contains data
+    unsigned int  pagebuf_dirty:1;	// data was modified
+    int		  last;			// when the page was last visited
 };
 
 typedef struct _AFFILE {
@@ -132,8 +140,9 @@ typedef struct _AFFILE {
     /* Page buffer cache */
     struct aff_pagebuf *pb;		// the current page buffer
     struct aff_pagebuf *pbcache;	// array of pagebufs
-    int    num_pbufs;			// number of pagebufs; default is 1
-    int	   afftime;			// for updating last
+    int		num_pbufs;	   // number of pagebufs; default is 1
+    int		afftime;		// for updating last
+    int64	cur_page;		// used by vnode_raw and vnode_ewf to fake pages
 
     int		  debug;		// for debugging, of course
     unsigned int  badflag_set:1;	// is badflag set?
@@ -199,14 +208,6 @@ struct affcallback_info {
     int compression_level;		// v1: compression level
 };
 
-
-
-
-/* AFF Log Mask */
-#define AF_LOG_af_seek  0x0001
-#define AF_LOG_af_write 0x0002
-#define AF_LOG_af_close 0x0004
-
 /* Utility Functions */
 
 #ifdef __cplusplus
@@ -265,7 +266,7 @@ int	af_is_badsector(AFFILE *af,unsigned char *buf); // 0 if not, 1 if it is, -1
 #define AF_IDENTIFY_EVD 4		// file is a .E01 file when there are more files following
 #define AF_IDENTIFY_SPLIT_RAW 5		// file is a split raw file
 #define AF_IDENTIFY_AFM 6             // file is raw file with metadata
-
+#define AF_IDENTIFY_EWF 7		// libewf
 
 
 #define AF_IDENTIFY_ERR -1		// error encountered on identify
@@ -279,8 +280,9 @@ const char *af_filename(AFFILE *af);	// returns the filename of an open stream.
 int	    af_identify(AFFILE *af);	// returns type of AFFILE pointer
 
 /* Accessor Functions */
-int64	    af_imagesize(AFFILE *af);	// byte # of last mapped byte in image, or size of device;
+int64	    af_get_imagesize(AFFILE *af);	// byte # of last mapped byte in image, or size of device;
 					// returns -1 if error
+#define af_imagesize(af) af_get_imagesize(af) // backwards compatiability 
 int	    af_get_segq(AFFILE *af,const char *name,int64 *quad);/* Get/set 8-byte values */
 int	    af_update_segq(AFFILE *af,const char *name,int64 quad);
 
@@ -349,7 +351,6 @@ int64	af_segname_hash_page_number(const char *name,char *hash,int hashlen);
  *** AF segment names that you might be interested in...
  ****************************************************************/
 
-
 #define AF_IGNORE       ""		// ignore segments with 0-length name
 #define AF_DIRECTORY    "dir"		// the directory
 #define AF_RAW_IMAGE_FILE_EXTENSION "raw_image_file_extension"
@@ -382,8 +383,6 @@ int64	af_segname_hash_page_number(const char *name,char *hash,int hashlen);
 #define AF_COMPRESSION_ALG_ZLIB 1	// try to compress with zlib
 #define AF_COMPRESSION_ALG_LZMA 2	// try to compress with LZMA
 
-
-
 /* Flags for data pages */
 #define AF_PAGE_COMPRESSED      0x0001
 #define AF_PAGE_COMP_MAX        0x0002	// compressed at maximum; nice to know
@@ -456,6 +455,18 @@ int64	af_segname_hash_page_number(const char *name,char *hash,int hashlen);
 #define AFFLIB_TRACE        "AFFLIB_TRACE" // make "1" to send all AFFLIB activity to stdout; "2" for stderr
 extern FILE *af_trace;		// fd to trace to
 
+/****************************************************************
+ *** Not AFF functions at all, but placed here for convenience.
+ ****************************************************************/
+const char *af_hexbuf(char *dst,int dst_len,const unsigned char *bin,int bytes,int format_flag);
+
+/* af_hexbuf formats: */
+#define AF_HEXBUF_NO_SPACES 0
+#define AF_HEXBUF_SPACE2    0x0001	// space every 2 characters
+#define AF_HEXBUF_SPACE4    0x0002	// space every 4 characters
+#define AF_HEXBUF_UPPERCASE 0x1000	// uppercase
+
+
 /****************************************************************
  *** Internal implementation details below.
  ****************************************************************/
