@@ -55,7 +55,21 @@ void af_read_sizes(AFFILE *af)
     if(af_get_seg(af,AF_PAGESIZE,&af->image_pagesize,0,0)){
 	af_get_seg(af,AF_SEGSIZE_D,&af->image_pagesize,0,0); // try old name
     }
-    af_get_segq(af,AF_IMAGESIZE,(int64 *)&af->image_size);
+    if(af_get_segq(af,AF_IMAGESIZE,(int64 *)&af->image_size)){
+	/* Need to recover the image size */
+	char segname[AF_MAX_NAME_LEN];
+	unsigned long arg;
+	size_t datalen = 0;
+	af_rewind_seg(af);		//  start at the beginning
+	int64 highest_page = -1;
+	while(af_get_next_seg(af,segname,sizeof(segname),&arg,0,&datalen)==0){
+	    if(segname[0]==0) continue;	// ignore sector
+	    int64 pagenum = af_segname_page_number(segname);
+	    if(pagenum > highest_page) highest_page = pagenum;
+	}
+	af->image_size = af->image_pagesize * (highest_page+1);
+    }
+	
     af->image_size_in_file = af->image_size;
 
     af_get_seg(af,AF_SECTORSIZE,&af->image_sectorsize,0,0);
@@ -166,7 +180,7 @@ int af_get_page(AFFILE *af,int64 pagenum,unsigned char *data,size_t *bytes)
     unsigned long arg=0;
     size_t page_len=0;
 
-    if (af_trace) fprintf(af_trace,"af_get_page(%p,%"I64d",%p,%d)\n",af,pagenum,data,bytes);
+    if (af_trace) fprintf(af_trace,"af_get_page(%p,pagenum=%"I64d",buf=%p,bytes=%d)\n",af,pagenum,data,bytes);
 
     /* Find out the size of the segment and if it is compressed or not.
      * If we can't find it with new nomenclature, try the old one...
@@ -244,7 +258,8 @@ int af_get_page(AFFILE *af,int64 pagenum,unsigned char *data,size_t *bytes)
 #ifdef USE_LZMA
 	case AF_PAGE_COMP_ALG_LZMA:
 	    res = lzma_uncompress(data,bytes,compressed_data,compressed_data_len);
-	    if (af_trace) fprintf(af_trace,"   LZMA decompressed. %d bytes => %d bytes\n",compressed_data_len,*bytes);
+	    if (af_trace) fprintf(af_trace,"   LZMA decompressed page %"I64d". %d bytes => %d bytes\n",
+				  pagenum,compressed_data_len,*bytes);
 	    switch(res){
 	    case 0:break;		// OK
 	    case 1:(*af->error_reporter)("LZMA header error decompressing segment %"I64d"\n",pagenum);
@@ -301,11 +316,13 @@ int af_update_page(AFFILE *af,int64 pagenum,unsigned char *data,int datalen)
 {
     char segname_buf[32];
 
+#ifdef HAVE_OPENSSL_MD5_H
     /* Write out the hash of the page */
     unsigned char md5_buf[16];
     MD5(data,datalen,md5_buf);
     snprintf(segname_buf,sizeof(segname_buf),AF_PAGE_MD5,pagenum);
     af_update_seg(af,segname_buf,0,md5_buf,sizeof(md5_buf)); // ignore failure
+#endif
 
     /* Check for bypass */
     if(af->v->write){
@@ -400,9 +417,9 @@ int af_update_page(AFFILE *af,int64 pagenum,unsigned char *data,int datalen)
 		    if(af->compression_level == AF_COMPRESSION_MAX){
 			flag |= AF_PAGE_COMP_MAX; // useful to know it can't be better
 		    }
-		    acbi.compressed = 1;	// it was compressed
-		    if(af->w_callback) {acbi.phase = 2;(*af->w_callback)(&acbi);}
 		}
+		acbi.compressed = 1;	// it was compressed (or not compressed)
+		if(af->w_callback) {acbi.phase = 2;(*af->w_callback)(&acbi);}
 	    }
 
 	    if(cres==0 && destLen < af->image_pagesize){
@@ -439,16 +456,14 @@ int af_update_page(AFFILE *af,int64 pagenum,unsigned char *data,int datalen)
  *** Cache interface
  ****************************************************************/
 
-/* Design of the hybrid write-through, write-back buffer cache system:
+/* The page cache is a read/write cache.
  * 
- * Each AFFILE can have 2 buffers associated with it.
- * The buffer that we are currently working on is pointed to by af->bp.
- * When a buffer is "purged", af->bp is set to be NULL and none of the buffers
- * are in use. 
+ * Pages that are read are cached after they are decompressed.
+ * When new pages are fetched, we check the cache first to see if they are there; 
+ * if so, they are satsfied by the cache.
+ * 
+ * Modifications are written to the cache, then dumped to the disk. 
  * 
- * When there is a read, we first see if there is a buffer currently allocated.
- * The RAM is associted with the cache when t starts up.
- *
  * The cache is managed by two functions:
  * af_cache_flush(af) - (prevously af_purge) 
  *      - Makes sure that all dirty buffers are written.
@@ -466,19 +481,21 @@ int af_update_page(AFFILE *af,int64 pagenum,unsigned char *data,int datalen)
  *
  */
 
+static int cachetime = 0;
+
 
 int af_cache_flush(AFFILE *af)
 {
+    if(af_trace) fprintf(af_trace,"af_cache_flush()\n");
     int ret = 0;
     for(int i=0;i<af->num_pbufs;i++){
 	struct aff_pagebuf *p = &af->pbcache[i];
-	if(p->valid && p->dirty){
+	if(p->pagebuf_valid && p->pagebuf_dirty){
 	    if(af_update_page(af,p->pagenum,p->pagebuf,p->pagebuf_bytes)){
 		ret = -1;		// got an error; keep going, though
 	    }
-	    p->dirty = 0;
-	    if(af_cache_debug>0) fprintf(stderr,"AFFLIB: AF %p slot %d page %qd flushed.\n",
-					 af,i,p->pagenum);
+	    p->pagebuf_dirty = 0;
+	    if(af_trace) fprintf(af_trace,"af_cache_flush: slot %d page %qd flushed.\n",i,p->pagenum);
 	}
     }
     return ret;				// now return the error that I might have gotten
@@ -493,31 +510,40 @@ void af_cache_writethrough(AFFILE *af,int64 pagenum,const unsigned char *buf,int
 {
     for(int i=0;i<af->num_pbufs;i++){
 	struct aff_pagebuf *p = &af->pbcache[i];
-	if(p->valid && p->pagenum == pagenum){
-	    if(p->dirty){
+	if(p->pagenum_valid && p->pagenum == pagenum){
+	    if(p->pagebuf_dirty){
 		(*af->error_reporter)("af_cache_writethrough: overwriting page %"I64u".\n",pagenum);
+		exit(-1);		// this shouldn't happen
 	    }
 	    memcpy(p->pagebuf,buf,bufflen);
 	    memset(p->pagebuf+bufflen,0,af->image_pagesize-bufflen); // zero fill the rest
 	    af->bytes_memcpy += bufflen;
-	    p->dirty = 0;		// no longer dirty
+	    p->pagebuf_valid = 1;	// we have a copy of it now.
+	    p->pagebuf_dirty = 0;	// but it isn't dirty
+	    p->last = cachetime++;
 	}
     }
 }
 	
+#ifdef HAVE_MALLOC_H
+#include <malloc.h>
+#endif
+
 #ifndef HAVE_VALLOC
 #define valloc malloc
 #endif
 
 struct aff_pagebuf *af_cache_alloc(AFFILE *af,int64 pagenum)
 {
-  int i;
+    if(af_trace) fprintf(af_trace,"af_cache_alloc(%p,%"I64d")\n",af,pagenum);
     af_cache_flush(af);			// make sure nothing in cache is dirty
     /* See if this page is already in the cache */
-    for(i=0;i<af->num_pbufs;i++){
+    for(int i=0;i<af->num_pbufs;i++){
 	struct aff_pagebuf *p = &af->pbcache[i];
-	if(p->valid && p->pagenum==pagenum){
+	if(p->pagenum_valid && p->pagenum==pagenum){
 	    af->cache_hits++;
+	    if(af_trace) fprintf(af_trace,"  page %"I64d" satisfied fromcache\n",pagenum);
+	    p->last = cachetime++;
 	    return p;
 	}
     }
@@ -525,18 +551,28 @@ struct aff_pagebuf *af_cache_alloc(AFFILE *af,int64 pagenum)
     af->cache_misses++;
     int slot = -1;
     /* See if there is an empty slot in the cache */
-    for(i=0;i<af->num_pbufs;i++){
+    for(int i=0;i<af->num_pbufs;i++){
 	struct aff_pagebuf *p = &af->pbcache[i];
-	if(p->valid==0){
+	if(p->pagenum_valid==0){
 	    slot = i;
+	    if(af_trace) fprintf(af_trace,"  slot %d given to page %"I64d"\n",slot,pagenum);
 	    break;
 	}
     }
-    if(slot==-1){			// need to find a random slot
-	slot = random() % af->num_pbufs;
+    if(slot==-1){			
+	/* Find the oldest cache entry */
+	int oldest_i = 0;
+	int oldest_t = af->pbcache[0].last;
+	for(int i=1;i<af->num_pbufs;i++){
+	    if(af->pbcache[i].last < oldest_t){
+		oldest_t = af->pbcache[i].last;
+		oldest_i = i;
+	    }
+	}
+	slot = oldest_i;
+	if(af_trace) fprintf(af_trace,"  slot %d assigned to page %"I64d"\n",slot,pagenum);
     }
     /* take over this slot */
-    if(af_cache_debug>0) fprintf(stderr,"AFFLIB: AF %p slot %d assigned to page %qd.\n",af,slot,pagenum);
     struct aff_pagebuf *p = &af->pbcache[slot];
     if(p->pagebuf==0){
 	p->pagebuf = (unsigned char *)valloc(af->image_pagesize); // allocate to a page boundary
@@ -551,7 +587,17 @@ struct aff_pagebuf *af_cache_alloc(AFFILE *af,int64 pagenum)
     }
     memset(p->pagebuf,0,af->image_pagesize); // clean object reuse
     p->pagenum = pagenum;
-    p->valid = 1;
+    p->pagenum_valid = 1;
+    p->pagebuf_valid = 0;
+    p->pagebuf_dirty = 0;
+    p->last = cachetime++;
+    if(af_trace){
+	fprintf(af_trace,"   current pages in cache: ");
+	for(int i=0;i<af->num_pbufs;i++){
+	    fprintf(af_trace," %"I64d,af->pbcache[i].pagenum);
+	}
+	fprintf(af_trace,"\n");
+    }
     return p;
 }
 
