@@ -1,20 +1,48 @@
 #include "config.h"
 
-#if defined(HAVE_LIBCURL) && defined(HAVE_LIBEXPAT)
+#include <stdio.h>
+#include <errno.h>
+#include <stdlib.h>
+
+#ifdef USE_S3
+
+#ifdef HAVE_SYS_PARAM_H
+#include <sys/param.h>
+#endif
+
+#if (defined(__FreeBSD_version)) && (__FreeBSD_version<500000) && (!defined(BAD_STL))
+#define BAD_STL
+#endif
+
+
 #include "s3_glue.h"
 #include "curl/curl.h"
 #include "base64.h"
 
 #include <expat.h>
-
-#include <errno.h>
-#include <stdio.h>
 #include <time.h>
-
 #include <netinet/in.h>
+
+#ifdef HAVE_OPENSSL_MD5_H
 #include <openssl/md5.h>
+#endif
+
+#ifdef HAVE_OPENSSL_HMAC_H
 #include <openssl/hmac.h>
+#endif
+
+#ifdef HAVE_ERR_H
 #include <err.h>
+#endif
+
+#ifdef HAVE_CTYPE_H
+#include <ctype.h>
+#endif
+
+#if !defined(HAVE_OPENSSL_MD5_H)
+#error S3 support requires MD5 support
+#endif
+
 
 
 int s3_debug = 0;
@@ -37,7 +65,6 @@ using namespace s3;
 
 const char *aws_access_key_id;
 const char *aws_secret_access_key;
-
 const char *aws_base_url = "http://s3.amazonaws.com/";
 
 /* Simson's S3 implementation in C++.
@@ -311,6 +338,13 @@ static string quote_plus(string &url)
     return eurl;
 }
 
+#ifndef HAVE_ISDIGIT
+static int isdigit(char ch)
+{
+    return ch>='0' && ch<='9';
+}
+#endif
+
 static int hexval(int ch) { return (isdigit(ch) ? ch-'0' : ch-'a'+10);}
 
 /*
@@ -339,7 +373,6 @@ class response_buffer *request(string method,string path,string query,time_t exp
 			       const char *sendbuf,size_t sendbuflen,
 			       const s3headers *extraheaders)
 {
-
     /* Note: this function is not threadsafe */
     static bool curl_initted = false;
     if(!curl_initted){
@@ -415,6 +448,11 @@ class response_buffer *request(string method,string path,string query,time_t exp
 	    curl_easy_setopt(c,CURLOPT_NOBODY,1);
 	}
 
+	/* Queries that take longer than an hour should timeout */
+	curl_easy_setopt(c,CURLOPT_TIMEOUT,60*60);
+
+	/* Disable DNS cache */
+	curl_easy_setopt(c,CURLOPT_DNS_CACHE_TIMEOUT,0); // per amazon specification
 	curl_easy_setopt(c,CURLOPT_WRITEFUNCTION,buffer_write);
 	curl_easy_setopt(c,CURLOPT_WRITEDATA,b); // fourth argument
 	curl_easy_setopt(c,CURLOPT_URL,url.c_str());
@@ -474,6 +512,7 @@ class response_buffer *request(string method,string path,string query,time_t exp
 	}
     } while(b->result==500 && ++retry_count<s3_retry_max);
 
+    if(b->result==404) errno=ENOENT;
 
     /* Pull out the headers */
     char *line,*brkt;
@@ -687,4 +726,11 @@ void s3_audit(int i)
 	fprintf(stderr,"S3 bytes read: %qu\n",s3_bytes_read);
     }
 }
-#endif
+#else
+int main(int argc,char **argv)
+{
+    fprintf(stderr,"S3 support is not enabled.\n");
+    exit(1);
+}
+
+#endif /* USE_S3 */
