@@ -397,6 +397,10 @@ static int aff_del_seg(AFFILE *af,const char *segname)
 
 
 
+#ifdef HAVE_OPENSSL_RAND_H
+#include <openssl/rand.h>
+#endif
+
 /* 
  * af_make_badflag:
  * Create a randomized bag flag and
@@ -405,8 +409,15 @@ static int aff_del_seg(AFFILE *af,const char *segname)
  */
 int af_make_badflag(AFFILE *af)
 {
+#ifdef HAVE_OPENSSL_RAND_H
     RAND_pseudo_bytes(af->badflag,af->image_sectorsize);
     strcpy((char *)af->badflag,"BAD SECTOR");
+#else
+    for(int i=0;i<af->image_sectorsize;i++){
+      af->badflag[i] = rand() & 0xff;
+    }
+#endif
+
     af->badflag_set = 1;
 
     if(af_update_seg(af,AF_BADFLAG,0,af->badflag,af->image_sectorsize)){
@@ -460,8 +471,8 @@ static int aff_open(AFFILE *af)
 
     /* Open the FILE  for the AFFILE */
     char strflag[8];
-    strcpy(strflag,"r");		// we have to be able to read
-    if(af->openflags & O_RDWR) 	strcpy(strflag,"w+"); 
+    strcpy(strflag,"rb");		// we have to be able to read
+    if(af->openflags & O_RDWR) 	strcpy(strflag,"w+b"); 
 
     af->aseg = fdopen(fd,strflag);
     if(!af->aseg){
