@@ -12,7 +12,6 @@ struct raw_private {
     /* For Raw files */
     FILE *raw;				// if it is a raw file
     int raw_popen;			// opened with popen
-    uint64 cur_page;			// current page number...
 };
 
 static inline struct raw_private *RAW_PRIVATE(AFFILE *af)
@@ -53,7 +52,7 @@ static int raw_open(AFFILE *af)
     if(af->fname) rp->raw=fopen(af->fname,"rb");
     af->image_size = raw_filesize(af);
     af->image_pagesize = RAW_PAGESIZE;
-    rp->cur_page = 0;
+    af->cur_page = 0;
     return 0;
 }
 
@@ -65,7 +64,7 @@ int raw_freopen(AFFILE *af,FILE *file)
     rp->raw = file;
     af->image_size = raw_filesize(af);
     af->image_pagesize = RAW_PAGESIZE;
-    rp->cur_page = 0;
+    af->cur_page = 0;
     return 0;
 }
 
@@ -109,9 +108,19 @@ static int raw_get_seg(AFFILE *af,const char *name,
 {
     struct raw_private *rp = RAW_PRIVATE(af);
 
-    /* Right now, this only supports page segments */
     int64 segnum = af_segname_page_number(name);
-    if(segnum<0) return -1;		// unknown segment number
+    if(segnum<0){
+	/* See if PAGESIZE or IMAGESIZE is being requested; we can fake those */
+	if(strcmp(name,AF_PAGESIZE)==0 && arg){*arg = af->image_pagesize;return 0;}
+	if(strcmp(name,AF_IMAGESIZE)==0 && data && *datalen>=8){
+	    struct aff_quad q;
+	    q.low = htonl((unsigned long)(af->image_size & 0xffffffff));
+	    q.high = htonl((unsigned long)(af->image_size >> 32));
+	    memcpy(data,&q,8);
+	    return 0;
+	}
+	return -1;		// don't know how to fake this
+    }
 
     fflush(rp->raw);			// make sure that any buffers are flushed
 
@@ -195,7 +204,7 @@ static int raw_vstat(AFFILE *af,struct af_vnode_info *vni)
 	}
     }
     vni->supports_compression = 0;
-    vni->has_pages = 1;
+    vni->has_pages = 1;			
 
     if(rp->raw_popen){
 	/* popen files require special handling */
@@ -208,28 +217,25 @@ static int raw_vstat(AFFILE *af,struct af_vnode_info *vni)
 
 static int raw_rewind_seg(AFFILE *af)
 {
-    struct raw_private *rp = RAW_PRIVATE(af);
-    
-    rp->cur_page = 0;
+    af->cur_page = 0;
     return 0;
 }
 
 
 static int raw_get_next_seg(AFFILE *af,char *segname,size_t segname_len,unsigned long *arg,
-			unsigned char *data,size_t *datalen_)
+			unsigned char *data,size_t *datalen)
 {
-    struct raw_private *rp = RAW_PRIVATE(af);
-    char pagename[AF_MAX_NAME_LEN];		//
     
     /* See if we are at the end of the "virtual" segment list */
-    if((rp->cur_page)*af->image_pagesize >= af->image_size) return -1;
+    if(af->cur_page * af->image_pagesize >= af->image_size) return -1;
 
     /* Make the segment name */
+    char pagename[AF_MAX_NAME_LEN];		//
     memset(pagename,0,sizeof(pagename));
-    snprintf(pagename,sizeof(pagename),AF_PAGE,rp->cur_page++);
+    snprintf(pagename,sizeof(pagename),AF_PAGE,af->cur_page++);
 
     /* Get the segment, if we can */
-    int r = raw_get_seg(af,pagename,arg,data,datalen_);
+    int r = raw_get_seg(af,pagename,arg,data,datalen);
 
     /* If r==0 and there is room for copying in the segment name, return it */
     if(r==0){
