@@ -51,6 +51,14 @@
 #include <ctype.h>
 #endif
 
+#ifndef HAVE_ISDIGIT
+static int isdigit(char ch)
+{
+    return ch>='0' && ch<='9';
+}
+#endif
+
+
 
 /* split raw file implementation with optional metadata support */
 struct split_raw_private {
@@ -186,7 +194,7 @@ static int split_raw_open_internal(AFFILE *af, uint64 *image_size)
     int fd;
     struct stat sb;
 
-    fd = open(srp->first_raw_fname, af->openflags, 0666);
+    fd = open(srp->first_raw_fname, af->openflags|O_BINARY, 0666);
     if (fd < 0) {
       (*af->error_reporter)("split_raw_open_internal: open(%s): ",af->fname);
 	return -1;
@@ -224,7 +232,7 @@ static int split_raw_open_internal(AFFILE *af, uint64 *image_size)
 	    return -1;
 	}
 	fd = open(srp->next_raw_fname,
-		   af->openflags & O_RDWR ? O_RDWR : O_RDONLY);
+		   af->openflags & O_RDWR ? (O_RDWR|O_BINARY) : (O_RDONLY|O_BINARY));
 
 	if (fd < 0) {
 	    if (errno != ENOENT) {
@@ -377,7 +385,7 @@ int split_raw_write_internal2(AFFILE *af, unsigned char *buf, uint64 pos,int cou
 	if (af->maxsize) {	// do we need to possibly split into multiple file writes?
 	    /* Figure out which file number we will need to write to... */
 	    if (pos >= (af->maxsize * srp->num_raw_files)) {
-		int fd = open(srp->next_raw_fname, O_RDWR | O_CREAT | O_EXCL, 0666);
+		int fd = open(srp->next_raw_fname, O_RDWR | O_CREAT | O_EXCL | O_BINARY, 0666);
 		if (fd < 0) {
 		  (*af->error_reporter)("split_raw_write: open(%s): ",af->fname);
 		    if (ret) return ret;
