@@ -16,9 +16,16 @@
 
 #ifdef HAVE_CURSES_H
 #include <curses.h>
+#endif
+
+#ifdef HAVE_TERM_H
 #include <term.h>
 #endif
 
+#ifdef HAVE_NCURSES_TERM_H
+#include <ncurses/term.h>
+#endif
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -128,7 +135,7 @@ void usage()
     printf("      -O dir   -- use 'dir' as the output directory\n");
     printf("      -o file  -- output to 'file' (can only convert one at a time)\n");
     printf("                  File is AFF is file ends .aff; otherwise assumes raw.\n");
-    printf("      -Xn      -- Set compression to n; default is 7");
+    printf("      -Xn      -- Set compression to n; default is 7\n");
     printf("      -L       -- Use the LZMA compression algorithm (better but slower)\n");
     
     printf("\nRaw output options:\n");
@@ -427,11 +434,11 @@ int convert(const char *infile,char *outfile)
 	    unsigned char *data = (unsigned char *)malloc(image_pagesize);
 	    size_t data_len = image_pagesize;
 	    if(af_get_page(a_in,pagenum,data,&data_len)){
-		err(1,"af_get_page(file=%s,page=%"I64d,
+		err(1,"af_get_page(file=%s,page=%"I64d")",
 		    af_filename(a_in),pagenum);
 	    }
 	    if(af_update_page(a_out,pagenum,data,data_len)){
-		err(1,"af_update_page(file=%s,page=%"I64d,
+		err(1,"af_update_page(file=%s,page=%"I64d")",
 		    af_filename(a_out),pagenum);
 	    }
 	    
@@ -557,7 +564,7 @@ int main(int argc,char **argv)
     char *outfile = 0;
     int ch;
 
-#ifdef HAVE_CURSES_H
+#ifdef HAVE_LIBNCURSES
     setupterm((char *)0,1,(int *)0);
 #endif
 
