@@ -1,12 +1,14 @@
 /*
  * The Sleuth Kit
  * 
- * $Date: 2006/12/07 16:25:38 $
+ * $Date: 2007/04/04 18:48:43 $
  */
 #ifndef _AUX_LIB_H
 #define _AUX_LIB_H
 
 #include <stdio.h>
+#include <stdlib.h>
+
 #include "tsk_os.h"
 
 #ifdef __cplusplus
@@ -14,14 +16,14 @@ extern "C" {
 #endif
 
 
-    extern char *mymalloc(size_t);
-    extern char *myrealloc(char *, size_t);
-    extern char *mystrdup(const char *);
+    extern char *tsk_malloc(size_t);
+    extern char *tsk_realloc(char *, size_t);
+    extern char *tsk_strdup(const char *);
 
-    extern char *split_at(char *, int);
-    extern char *split_at_right(char *, int);
+    extern char *tsk_split_at(char *, int);
+    extern char *tsk_split_at_right(char *, int);
 
-/* printf macros - if the OS doesnot have inttypes.h yet */
+/* printf macros - if the OS does not have inttypes.h yet */
 
 #ifndef PRIx64
 #define PRIx64 "llx"
@@ -122,11 +124,11 @@ extern "C" {
 #define PRIdPNUM	PRId32
 
 
-    extern void print_version(FILE *);
+    extern void tsk_print_version(FILE *);
     extern char *tskGetVersion();
-    extern SSIZE_T parse_offset(TSK_TCHAR *);
-    extern int parse_inum(const TSK_TCHAR * str, INUM_T *, uint32_t *,
-	uint16_t *, int *);
+    extern SSIZE_T tsk_parse_offset(TSK_TCHAR *);
+    extern int tsk_parse_inum(const TSK_TCHAR * str, INUM_T *, uint32_t *,
+        uint16_t *, int *);
 
 
 /* 
@@ -137,22 +139,22 @@ extern "C" {
 #define TSK_BIG_ENDIAN	0x02
 
 /* macros to read in multi-byte fields
- *  * file system is an array of 8-bit values, not 32-bit values
- *   */
-    extern uint8_t guess_end_u16(uint8_t *, uint8_t *, uint16_t);
-    extern uint8_t guess_end_u32(uint8_t *, uint8_t *, uint32_t);
+ * file system is an array of 8-bit values, not 32-bit values
+ */
+    extern uint8_t tsk_guess_end_u16(uint8_t *, uint8_t *, uint16_t);
+    extern uint8_t tsk_guess_end_u32(uint8_t *, uint8_t *, uint32_t);
 
 /* 16-bit values */
-#define getu16(flag, x)   \
+#define tsk_getu16(flag, x)   \
     (uint16_t)(((flag) & TSK_LIT_ENDIAN) ? \
 	  (((uint8_t *)(x))[0] + (((uint8_t *)(x))[1] << 8)) :    \
 	  (((uint8_t *)(x))[1] + (((uint8_t *)(x))[0] << 8)) )
 
-#define gets16(flag, x)	\
-	((int16_t)getu16(flag, x))
+#define tsk_gets16(flag, x)	\
+	((int16_t)tsk_getu16(flag, x))
 
 /* 32-bit values */
-#define getu32(flag, x)	\
+#define tsk_getu32(flag, x)	\
 	(uint32_t)( ((flag) & TSK_LIT_ENDIAN)  ?	\
      ((((uint8_t *)(x))[0] <<  0) + \
 	  (((uint8_t *)(x))[1] <<  8) + \
@@ -164,10 +166,10 @@ extern "C" {
 	  (((uint8_t *)(x))[1] << 16) + \
 	  (((uint8_t *)(x))[0] << 24) ) )
 
-#define gets32(flag, x)	\
-	((int32_t)getu32(flag, x))
+#define tsk_gets32(flag, x)	\
+	((int32_t)tsk_getu32(flag, x))
 
-#define getu48(flag, x)   \
+#define tsk_getu48(flag, x)   \
 	(uint64_t)( ((flag) & TSK_LIT_ENDIAN)  ?	\
       ((uint64_t) \
 	  ((uint64_t)((uint8_t *)(x))[0] <<  0)+ \
@@ -186,7 +188,7 @@ extern "C" {
       ((uint64_t)((uint8_t *)(x))[0] << 40)) )
 
 
-#define getu64(flag, x)   \
+#define tsk_getu64(flag, x)   \
 	(uint64_t)( ((flag) & TSK_LIT_ENDIAN)  ?	\
       ((uint64_t) \
 	  ((uint64_t)((uint8_t *)(x))[0] << 0)  + \
@@ -208,13 +210,24 @@ extern "C" {
       ((uint64_t)((uint8_t *)(x))[1] << 48) + \
       ((uint64_t)((uint8_t *)(x))[0] << 56)) )
 
-#define gets64(flag, x)	\
-	((int64_t)getu64(flag, x))
+#define tsk_gets64(flag, x)	\
+	((int64_t)tsk_getu64(flag, x))
+
+
+
+
+
+/*********** RETURN VALUES ************/
+
+    /* Flags for the return value of walk actions */
+#define TSK_WALK_CONT	0x0
+#define TSK_WALK_STOP	0x1
+#define TSK_WALK_ERROR	0x2
 
 
 
 /************ ERROR HANDLING *************/
-    extern int verbose;
+    extern int tsk_verbose;
 
 #define TSK_ERRSTR_L	512
 #define TSK_ERRSTR_PR_L	(TSK_ERRSTR_L << 2)
@@ -232,6 +245,7 @@ extern "C" {
 #define TSK_ERR_IMG	0x02000000
 #define TSK_ERR_MM	0x04000000
 #define TSK_ERR_FS	0x08000000
+#define TSK_ERR_HDB	0x10000000
 #define TSK_ERR_MASK	0x00ffffff
 
 #define TSK_ERR_AUX_MALLOC	(TSK_ERR_AUX | 0)
@@ -279,24 +293,39 @@ extern "C" {
 #define TSK_ERR_FS_MAX		16
 
 
+#define TSK_ERR_HDB_UNKTYPE     (TSK_ERR_HDB | 0)
+#define TSK_ERR_HDB_UNSUPTYPE   (TSK_ERR_HDB | 1)
+#define TSK_ERR_HDB_READDB	(TSK_ERR_HDB | 2)
+#define TSK_ERR_HDB_READIDX	(TSK_ERR_HDB | 3)
+#define TSK_ERR_HDB_ARG		(TSK_ERR_HDB | 4)
+#define TSK_ERR_HDB_WRITE	(TSK_ERR_HDB | 5)
+#define TSK_ERR_HDB_CREATE	(TSK_ERR_HDB | 6)
+#define TSK_ERR_HDB_DELETE      (TSK_ERR_HDB | 7)
+#define TSK_ERR_HDB_MISSING     (TSK_ERR_HDB | 8)
+#define TSK_ERR_HDB_PROC        (TSK_ERR_HDB | 9)
+#define TSK_ERR_HDB_OPEN        (TSK_ERR_HDB | 10)
+#define TSK_ERR_HDB_CORRUPT     (TSK_ERR_HDB | 11)
+#define TSK_ERR_HDB_MAX		12
+
+
 
 /************* DATA BUF ******************/
-    typedef struct DATA_BUF DATA_BUF;
+    typedef struct TSK_DATA_BUF TSK_DATA_BUF;
 
-    struct DATA_BUF {
-	char *data;		/* buffer memory */
-	size_t size;		/* buffer size */
-	size_t used;		/* amount of space used */
-	DADDR_T addr;		/* start block */
+    struct TSK_DATA_BUF {
+        char *data;             /* buffer memory */
+        size_t size;            /* buffer size */
+        size_t used;            /* amount of space used */
+        DADDR_T addr;           /* start block */
     };
 
-    extern DATA_BUF *data_buf_alloc(size_t);
-    extern void data_buf_free(DATA_BUF *);
+    extern TSK_DATA_BUF *tsk_data_buf_alloc(size_t);
+    extern void tsk_data_buf_free(TSK_DATA_BUF *);
 
 
     // basic check to see if a Unicode file has been included 
     // in an app that is using this as a library
-#ifndef UNI_REPLACEMENT_CHAR
+#ifndef TSK_UNI_REPLACEMENT_CHAR
 
 /**************** UNICODE *******************/
 /*
@@ -353,7 +382,7 @@ extern "C" {
 	sourceEnd, targetEnd - respectively pointers to the ends of the
 		two buffers, for overflow checking only.
 
-    These conversion functions take a ConversionFlags argument. When this
+    These conversion functions take a TSKConversionFlags argument. When this
     flag is set to strict, both irregular sequences and isolated surrogates
     will cause an error.  When the flag is set to lenient, both irregular
     sequences and isolated surrogates are converted.
@@ -368,8 +397,8 @@ extern "C" {
     they constitute an error.
 
     Output parameters:
-	The value "sourceIllegal" is returned from some routines if the input
-	sequence is malformed.  When "sourceIllegal" is returned, the source
+	The value "TSKsourceIllegal" is returned from some routines if the input
+	sequence is malformed.  When "TSKsourceIllegal" is returned, the source
 	value will point to the illegal value that caused the problem. E.g.,
 	in UTF-8 when a sequence is malformed, it points to the start of the
 	malformed sequence.  
@@ -391,40 +420,40 @@ extern "C" {
 #define TSK_IS_CNTRL(x) \
     (((x) < 0x20) && ((x) >= 0x00))
 
-    typedef unsigned long UTF32;	/* at least 32 bits */
-    typedef unsigned short UTF16;	/* at least 16 bits */
-    typedef unsigned char UTF8;	/* typically 8 bits */
-    typedef unsigned char Boolean;	/* 0 or 1 */
+    typedef unsigned long UTF32;        /* at least 32 bits */
+    typedef unsigned short UTF16;       /* at least 16 bits */
+    typedef unsigned char UTF8; /* typically 8 bits */
+    typedef unsigned char Boolean;      /* 0 or 1 */
 
 /* Some fundamental constants */
-#define UNI_REPLACEMENT_CHAR (UTF32)0x0000FFFD
-#define UNI_MAX_BMP (UTF32)0x0000FFFF
-#define UNI_MAX_UTF16 (UTF32)0x0010FFFF
-#define UNI_MAX_UTF32 (UTF32)0x7FFFFFFF
-#define UNI_MAX_LEGAL_UTF32 (UTF32)0x0010FFFF
+#define TSK_UNI_REPLACEMENT_CHAR (UTF32)0x0000FFFD
+#define TSK_UNI_MAX_BMP (UTF32)0x0000FFFF
+#define TSK_UNI_MAX_UTF16 (UTF32)0x0010FFFF
+#define TSK_UNI_MAX_UTF32 (UTF32)0x7FFFFFFF
+#define TSK_UNI_MAX_LEGAL_UTF32 (UTF32)0x0010FFFF
 
     typedef enum {
-	conversionOK,		/* conversion successful */
-	sourceExhausted,	/* partial character in source, but hit end */
-	targetExhausted,	/* insuff. room in target for conversion */
-	sourceIllegal		/* source sequence is illegal/malformed */
-    } ConversionResult;
+        TSKconversionOK,        /* conversion successful */
+        TSKsourceExhausted,     /* partial character in source, but hit end */
+        TSKtargetExhausted,     /* insuff. room in target for conversion */
+        TSKsourceIllegal        /* source sequence is illegal/malformed */
+    } TSKConversionResult;
 
     typedef enum {
-	strictConversion = 0,
-	lenientConversion
-    } ConversionFlags;
+        TSKstrictConversion = 0,
+        TSKlenientConversion
+    } TSKConversionFlags;
 
-    ConversionResult tsk_UTF8toUTF16(const UTF8 ** sourceStart,
-	const UTF8 * sourceEnd,
-	UTF16 ** targetStart, UTF16 * targetEnd, ConversionFlags flags);
+    TSKConversionResult tsk_UTF8toUTF16(const UTF8 ** sourceStart,
+        const UTF8 * sourceEnd,
+        UTF16 ** targetStart, UTF16 * targetEnd, TSKConversionFlags flags);
 
-    ConversionResult tsk_UTF16toUTF8(uint16_t, const UTF16 ** sourceStart,
-	const UTF16 * sourceEnd,
-	UTF8 ** targetStart, UTF8 * targetEnd, ConversionFlags flags);
+    TSKConversionResult tsk_UTF16toUTF8(uint16_t,
+        const UTF16 ** sourceStart, const UTF16 * sourceEnd,
+        UTF8 ** targetStart, UTF8 * targetEnd, TSKConversionFlags flags);
 
     Boolean tsk_isLegalUTF8Sequence(const UTF8 * source,
-	const UTF8 * sourceEnd);
+        const UTF8 * sourceEnd);
 
 #endif
 // getopt and windows stuff
@@ -443,15 +472,105 @@ extern "C" {
 
     typedef struct TSK_LIST TSK_LIST;
     struct TSK_LIST {
-	TSK_LIST *next;
-	uint64_t key;
-	uint64_t len;
+        TSK_LIST *next;
+        uint64_t key;
+        uint64_t len;
     };
 
     extern uint8_t tsk_list_add(TSK_LIST ** list, uint64_t key);
     extern uint8_t tsk_list_find(TSK_LIST * list, uint64_t key);
     extern void tsk_list_free(TSK_LIST * list);
 
+
+
+
+/*************************** HASH STUFF *********************/
+
+/* Copyright (C) 1991-2, RSA Data Security, Inc. Created 1991. All
+rights reserved.
+
+License to copy and use this software is granted provided that it
+is identified as the "RSA Data Security, Inc. MD5 Message-Digest
+Algorithm" in all material mentioning or referencing this software
+or this function.
+
+License is also granted to make and use derivative works provided
+that such works are identified as "derived from the RSA Data
+Security, Inc. MD5 Message-Digest Algorithm" in all material
+mentioning or referencing the derived work.
+
+RSA Data Security, Inc. makes no representations concerning either
+the merchantability of this software or the suitability of this
+software for any particular purpose. It is provided "as is"
+without express or implied warranty of any kind.
+
+These notices must be retained in any copies of any part of this
+documentation and/or software.
+ */
+
+
+/* POINTER defines a generic pointer type */
+    typedef unsigned char *POINTER;
+
+/* UINT2 defines a two byte word */
+//typedef unsigned short int UINT2;
+    typedef uint16_t UINT2;
+
+/* UINT4 defines a four byte word */
+    typedef uint32_t UINT4;
+
+/*
+#ifdef __alpha
+typedef unsigned int UINT4;
+#else
+typedef unsigned long int UINT4;
+#endif
+*/
+
+
+/* Added for sha1 */
+/* BYTE defines a unsigned character */
+    typedef uint8_t BYTE;
+
+#ifndef TRUE
+#define FALSE 0
+#define TRUE  ( !FALSE )
+#endif                          /* TRUE */
+
+
+
+/* MD5 context. */
+    typedef struct {
+        UINT4 state[4];         /* state (ABCD) */
+        UINT4 count[2];         /* number of bits, modulo 2^64 (lsb first) */
+        unsigned char buffer[64];       /* input buffer */
+    } MD5_CTX;
+
+    void MD5Init(MD5_CTX *);
+    void MD5Update(MD5_CTX *, unsigned char *, unsigned int);
+    void MD5Final(unsigned char[16], MD5_CTX *);
+
+
+
+/* sha.h */
+
+/* The structure for storing SHS info */
+
+    typedef struct {
+        UINT4 digest[5];        /* Message digest */
+        UINT4 countLo, countHi; /* 64-bit bit count */
+        UINT4 data[16];         /* SHS data buffer */
+        int Endianness;
+    } SHA_CTX;
+
+/* Message digest functions */
+
+    void SHAInit(SHA_CTX *);
+    void SHAUpdate(SHA_CTX *, BYTE * buffer, int count);
+    void SHAFinal(BYTE * output, SHA_CTX *);
+
+
+
 #ifdef __cplusplus
 }
 #endif
