@@ -2,7 +2,7 @@
  * The Sleuth Kit - Add on for EWF image support
  * Eye Witness Compression Format Support
  *
- * $Date: 2006/06/19 21:54:26 $
+ * $Date: 2007/03/20 21:54:53 $
  *
  * Joachim Metz <metz@studenten.net>
  * Copyright (c) 2006 Joachim Metz.  All rights reserved 
@@ -21,13 +21,13 @@
 extern "C" {
 #endif
 
-    extern IMG_INFO *ewf_open(int, const char **, IMG_INFO *);
+    extern TSK_IMG_INFO *ewf_open(int, const char **, TSK_IMG_INFO *);
 
     typedef struct IMG_EWF_INFO IMG_EWF_INFO;
     struct IMG_EWF_INFO {
-	IMG_INFO img_info;
-	LIBEWF_HANDLE *handle;
-	char *md5hash;
+        TSK_IMG_INFO img_info;
+        LIBEWF_HANDLE *handle;
+        char *md5hash;
     };
 
 #ifdef __cplusplus
