@@ -1,12 +1,18 @@
 # Script for compiling libewf
 # as part of The Sleuth Kit
 #
-# $Date: 2006/12/06 17:32:51 $
+# $Date: 2007/03/29 20:41:55 $
 #
 cd libewf
 if (test -f './Makefile') then
     true
 else
+    (touch Makefile.am && \
+     touch configure.in && \
+     touch aclocal.m4 && \
+     touch Makefile.in && \
+     touch config.h.in && \
+     touch configure) || (cd ..; exit $?)
     ./configure || (cd ..; exit $?)
 fi
 
