@@ -1,7 +1,7 @@
 /*
  * The Sleuth Kit
  *
- * $Date: 2006/09/06 20:40:02 $
+ * $Date: 2007/04/04 20:06:59 $
  *
  * Brian Carrier [carrier@sleuthkit.org]
  * Copyright (c) 2003-2005 Brian Carrier.  All rights reserved
@@ -17,26 +17,29 @@
 
 typedef struct {
     char *name;
-    char code;
+    TSK_MM_INFO_TYPE_ENUM code;
     char *comment;
 } MM_TYPES;
 
 
 MM_TYPES mm_open_table[] = {
-    {"dos", MM_DOS, "DOS-based partitions [Windows, Linux, etc.]"},
-    {"mac", MM_MAC, "MAC partitions"},
-    {"bsd", MM_BSD, "BSD Disklabels [FreeBSD, OpenBSD, NetBSD]"},
-    {"sun", MM_SUN, "Sun Volume Table of Contents (Solaris)"},
-    {"gpt", MM_GPT, "GUID Partition Table (EFI)"},
+    {"dos", TSK_MM_INFO_TYPE_DOS,
+            "DOS-based partitions [Windows, Linux, etc.]"},
+    {"mac", TSK_MM_INFO_TYPE_MAC, "MAC partitions"},
+    {"bsd", TSK_MM_INFO_TYPE_BSD,
+            "BSD Disklabels [FreeBSD, OpenBSD, NetBSD]"},
+    {"sun", TSK_MM_INFO_TYPE_SUN,
+            "Sun Volume Table of Contents (Solaris)"},
+    {"gpt", TSK_MM_INFO_TYPE_GPT, "GUID Partition Table (EFI)"},
     {0},
 };
 
 /* parse the string and return the value 
  *
- * Returns MM_UNSUPP if string cannot be parsed
+ * Returns TSK_MM_INFO_TYPE_UNSUPP if string cannot be parsed
  * */
-char
-mm_parse_type(const TSK_TCHAR * str)
+TSK_MM_INFO_TYPE_ENUM
+tsk_mm_parse_type(const TSK_TCHAR * str)
 {
     char tmp[16];
     int i;
@@ -44,25 +47,25 @@ mm_parse_type(const TSK_TCHAR * str)
 
     // convert to char
     for (i = 0; i < 15 && str[i] != '\0'; i++) {
-	tmp[i] = (char) str[i];
+        tmp[i] = (char) str[i];
     }
     tmp[i] = '\0';
 
     for (types = mm_open_table; types->name; types++) {
-	if (strcmp(tmp, types->name) == 0) {
-	    return types->code;
-	}
+        if (strcmp(tmp, types->name) == 0) {
+            return types->code;
+        }
     }
-    return MM_UNSUPP;
+    return TSK_MM_INFO_TYPE_UNSUPP;
 }
 
 void
-mm_print_types(FILE * hFile)
+tsk_mm_print_types(FILE * hFile)
 {
     MM_TYPES *types;
     tsk_fprintf(hFile, "Supported partition types:\n");
     for (types = mm_open_table; types->name; types++)
-	tsk_fprintf(hFile, "\t%s (%s)\n", types->name, types->comment);
+        tsk_fprintf(hFile, "\t%s (%s)\n", types->name, types->comment);
 }
 
 
@@ -72,12 +75,12 @@ mm_print_types(FILE * hFile)
  * Returns NULL if a match is not made
  * */
 char *
-mm_get_type(char mmtype)
+tsk_mm_get_type(TSK_MM_INFO_TYPE_ENUM mmtype)
 {
     MM_TYPES *types;
     for (types = mm_open_table; types->name; types++)
-	if (types->code == mmtype)
-	    return types->name;
+        if (types->code == mmtype)
+            return types->name;
 
     return NULL;
 }
